<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');

/**
 * Default Extension
 *
 * @package     Default module
 * @category    Modules
 * @author      Rein de Vries <info@reinos.nl>
 * @link        http://reinos.nl
 * @copyright   Copyright (c) 2013 Reinos.nl Internet Media
 */

/**
 * Include the config file
 */
require_once PATH_THIRD.'relationship_vars/config.php';

class Relationship_vars_ext {

    var $name           = REL_VARS_NAME;
    var $version        = REL_VARS_VERSION;
    var $description    = REL_VARS_DESCRIPTION;
    var $settings_exist = 'n';
    var $docs_url       = REL_VARS_DOCS;

    var $settings       = array();

    /**
     * Constructor
     *
     * @param   mixed   Settings array or empty string if none exist.
     */
    function __construct($settings = '')
    {
        $this->EE =& get_instance();

        $this->settings = $settings;
    }

    /**
     * Activate Extension
     *
     * This function enters the extension into the exp_extensions table
     *
     * @see http://codeigniter.com/user_guide/database/index.html for
     * more information on the db class.
     *
     * @return void
     */
    function activate_extension()
    {
        $data = array(
            'class'     => __CLASS__,
            'method'    => 'channel_entries_tagdata',
            'hook'      => 'channel_entries_tagdata',
            'settings'  => serialize($this->settings),
            'priority'  => 10,
            'version'   => $this->version,
            'enabled'   => 'y'
        );

        $this->EE->db->insert('extensions', $data);
    }

    /**
     * Update Extension
     *
     * This function performs any necessary db updates when the extension
     * page is visited
     *
     * @param string $current
     * @return  mixed   void on update / false if none
     */
    function update_extension($current = '')
    {
        if ($current == '' OR $current == $this->version)
        {
            return FALSE;
        }

        $this->EE->db->where('class', __CLASS__);
        $this->EE->db->update(
            'extensions',
            array('version' => $this->version)
        );
    }

    /**
     * Disable Extension
     *
     * This method removes information from the exp_extensions table
     *
     * @return void
     */
    function disable_extension()
    {
        $this->EE->db->where('class', __CLASS__);
        $this->EE->db->delete('extensions');
    }


    /**
     * channel_entries_tagdata_end
     *
     * set the vars
     *
     * @param $tagdata
     * @param $row
     * @param $channel
     * @return string
     * @internal param $
     */
    function channel_entries_tagdata($tagdata, $row, $channel)
    {
        // has this hook already been called?
        if (ee()->extensions->last_call)
        {
            $tagdata = ee()->extensions->last_call;
        }

        $tagdata = $this->create_tagdata_for_entry_id($row['entry_id'], $tagdata, $row);

        //return the tagdata
        return $tagdata;
    }

    // ----------------------------------------------------------------------

    /**
     * Parse the new vars
     *
     * @param string $entry_id
     * @param string $tagdata
     * @param array $row
     * @internal param $none
     * @return string
     */
    public function create_tagdata_for_entry_id($entry_id = '', $tagdata = '', $row = array())
    {
        //{rel_var:total_siblings}
        if(preg_match_all("/".LD."(.*?):total_siblings(.*?)".RD."/s", $tagdata, $all_matches))
        {
            foreach($all_matches[0] as $key=>$val)
            {
                //get the field to look
                $field = $all_matches[1][$key];

                //get the params
                $parameters = ee()->functions->assign_parameters($all_matches[2][$key]);

                //get the field settings channels
                $channel_ids = $this->get_field_settings($field);
                if($channel_ids !== false)
                {
                    $status = isset($parameters['status']) ? $parameters['status'] : 'open' ;
                    $result = $this->get_siblings($entry_id, $status, $channel_ids);
                    $tagdata = str_replace($all_matches[0][$key], $result->num_rows(), $tagdata);
                }
                else
                {
                    ee()->TMPL->log_item('Error: the "field" does not exists');
                }
            }
        }

        //{rel_var:has_siblings}{/rel_var:has_siblings}
        if(preg_match_all("/".LD."(\w+):has_siblings(.*?)".RD."(.*?)".LD."\/\w+:has_siblings".RD."/s", $tagdata, $all_matches))
        {
            foreach($all_matches[0] as $key=>$val)
            {
                //get the field to look
                $field = $all_matches[1][$key];

                //get the params
                $parameters = ee()->functions->assign_parameters($all_matches[2][$key]);

                //get the field settings channels
                $channel_ids = $this->get_field_settings($field);

                if($channel_ids !== false)
                {
                    $status = isset($parameters['status']) ? $parameters['status'] : 'open' ;
                    $result = $this->get_siblings($entry_id, $status, $channel_ids);

                    if($result->num_rows() > 0)
                    {
                        $tagdata = str_replace($all_matches[0][$key], $all_matches[3][$key], $tagdata);
                    }
                    else
                    {
                        $tagdata = str_replace($all_matches[0][$key], '', $tagdata);
                    }
                }
                else
                {
                    ee()->TMPL->log_item('Error: the "field" does not exists');
                }

            }
        }

        //{rel_var:no_siblings}{/rel_var:no_siblings}
        if(preg_match_all("/".LD."(\w+):no_siblings(.*?)".RD."(.*?)".LD."\/\w+:no_siblings".RD."/s", $tagdata, $all_matches))
        {
            foreach($all_matches[0] as $key=>$val)
            {
                //get the field to look
                $field = $all_matches[1][$key];

                //get the params
                $parameters = ee()->functions->assign_parameters($all_matches[2][$key]);

                //get the field settings channels
                $channel_ids = $this->get_field_settings($field);

                if($channel_ids !== false)
                {
                    $status = isset($parameters['status']) ? $parameters['status'] : 'open' ;
                    $result = $this->get_siblings($entry_id, $status, $channel_ids);

                    if($result->num_rows() == 0)
                    {
                        $tagdata = str_replace($all_matches[0][$key], $all_matches[3][$key], $tagdata);
                    }
                    else
                    {
                        $tagdata = str_replace($all_matches[0][$key], '', $tagdata);
                    }
                }
                else
                {
                    ee()->TMPL->log_item('Error: the "field" does not exists');
                }

            }
        }

        return $tagdata;
    }

    /**
     * @param $parent_id
     * @param string $status
     * @param int $channel_id
     * @return mixed
     */
    private function get_siblings($parent_id, $status = 'open', $channel_ids = array())
    {
        ee()->db->select('*');
        ee()->db->from('relationships rel');
        ee()->db->join('channel_titles c', 'rel.child_id = c.entry_id');
        ee()->db->where('rel.parent_id', $parent_id);
        ee()->db->where('c.status', $status);
        if(!empty($channel_ids))
        {
            ee()->db->where_in('c.channel_id', $channel_ids);
        }

        return ee()->db->get();
    }

    /**
     * @param $sibling_id
     * @param string $status
     * @param int $channel_id
     * @return mixed
     */
    private function get_parents($sibling_id, $status = 'open', $channel_id = 0)
    {
        ee()->db->select('*');
        ee()->db->from('relationships rel');
        ee()->db->join('channel_titles c', 'rel.parent_id = c.entry_id');
        ee()->db->where('rel.child_id', $sibling_id);
        ee()->db->where('c.status', $status);
        ee()->db->where('c.channel_id', $channel_id);
        return ee()->db->get();
    }

    /**
     * @param string $field_name
     * @internal param $sibling_id
     * @internal param string $status
     * @internal param int $channel_id
     * @return mixed
     */
    private function get_field_settings($field_name = '')
    {
        ee()->db->select('field_settings');
        ee()->db->from('channel_fields');
        ee()->db->where('field_name', $field_name);
        ee()->db->where('field_type', 'relationship');

        $result = ee()->db->get();

        if($result->num_rows() > 0)
        {
            $field_settings = $result->row();
            $field_settings = unserialize(base64_decode($field_settings->field_settings));

            if(isset($field_settings['channels']))
            {
                return $field_settings['channels'];
            }
        }

        return false;
    }

}
// END CLASS